package org.eclipse.e4.languages.internal.javascript.debug.launching;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IRegistryEventListener;
import org.eclipse.core.runtime.InvalidRegistryObjectException;
import org.eclipse.core.runtime.RegistryFactory;

public class ConnectorsManager implements IRegistryEventListener {

	private static final String JSDI_CONNECTORS = "org.eclipse.e4.languages.javascript.debug.jsdi.connectors"; //$NON-NLS-1$
	private static final String CONNECTOR = "connector"; //$NON-NLS-1$
	private static final String CLASS = "class"; //$NON-NLS-1$
	private List connectors;

	public ConnectorsManager() {
		RegistryFactory.getRegistry().addListener(this, JSDI_CONNECTORS);
	}

	public synchronized List getConnectors() {
		if (connectors != null)
			return connectors;

		connectors = new ArrayList();
		IExtension[] extensions = RegistryFactory.getRegistry().getExtensionPoint(JSDI_CONNECTORS).getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IExtension extension = extensions[i];
			try {
				IConfigurationElement[] elements = extension.getConfigurationElements();
				for (int j = 0; j < elements.length; j++) {
					IConfigurationElement element = elements[j];
					if (!CONNECTOR.equalsIgnoreCase(element.getName())) {
						// log it
						continue;
					}
					connectors.add(element.createExecutableExtension(CLASS));
				}
			} catch (InvalidRegistryObjectException e) {
				// skip this extension
			} catch (CoreException e) {
				// log it
				// skip this extension
			}
		}
		return connectors;
	}

	public void added(IExtension[] extensions) {
		connectors = null;
	}

	public void added(IExtensionPoint[] extensionPoints) {
		connectors = null;
	}

	public void removed(IExtension[] extensions) {
		connectors = null;
	}

	public void removed(IExtensionPoint[] extensionPoints) {
		connectors = null;
	}
}
