package org.eclipse.e4.languages.javascript.jsdi.rhino.connect;

import org.eclipse.e4.languages.javascript.jsdi.connect.Connector.IntegerArgument;

public class PortArgument implements IntegerArgument {

	private static final long serialVersionUID = -1954469572907116388L;
	private int port;

	public PortArgument(int port) {
		setValue(port);
	}

	public int intValue() {
		return port;
	}

	public boolean isValid(int intValue) {
		return intValue > 0;
	}

	public int max() {
		return Integer.MAX_VALUE;
	}

	public int min() {
		return 1;
	}

	public void setValue(int port) {
		if (!isValid(port))
			throw new IllegalArgumentException();
		this.port = port;
	}

	public String description() {
		return "The port on the host to attach to.";
	}

	public boolean isValid(String value) {
		try {
			int intValue = Integer.parseInt(value);
			return isValid(intValue);
		} catch (NumberFormatException e) {
			return false;
		}
	}

	public String label() {
		return "port";
	}

	public boolean mustSpecify() {
		return true;
	}

	public String name() {
		return "port";
	}

	public void setValue(String value) {
		int intValue = Integer.parseInt(value);
		setValue(intValue);
	}

	public String value() {
		return Integer.toString(port);
	}
}
